@@ -1,6 +1,6 @@
 /*
     DABlin - capital DAB experience
-    Copyright (C) 2015-2021 Stefan Pöschel
+    Copyright (C) 2015-2022 Stefan Pöschel
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -264,6 +264,7 @@ DABlinGTK::DABlinGTK(DABlinGTKOptions options) {
 	fic_change_service.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::FICChangeServiceEmitted));
 	pad_change_dynamic_label.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeDynamicLabelEmitted));
 	pad_change_slide.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADChangeSlideEmitted));
+	pad_file_progress.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::PADFileProgressEmitted));
 	do_rec_status_update.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::DoRecStatusUpdateEmitted));
 	do_datetime_sync.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::DoDateTimeSyncEmitted));
 	do_datetime_update.GetDispatcher().connect(sigc::mem_fun(*this, &DABlinGTK::DoDateTimeUpdateEmitted));
@@ -1075,12 +1076,13 @@ void DABlinGTK::FICChangeEnsembleEmitted() {
 	ensemble = fic_change_ensemble.Pop();
 
 	std::string charset_name;
-	std::string label = FICDecoder::ConvertLabelToUTF8(ensemble.label, &charset_name);
+	std::string label = Glib::Markup::escape_text(FICDecoder::ConvertLabelToUTF8(ensemble.label, &charset_name));
 
 	std::string tooltip_text =
 			"Short label: \"" + FICDecoder::DeriveShortLabelUTF8(label, ensemble.label.short_label_mask) + "\"\n"
 			"Label charset: " + charset_name + "\n"
-			"EId: " + StringTools::IntToHex(ensemble.eid, 4);
+			"EId: " + StringTools::IntToHex(ensemble.eid, 4) + "\n"
+			"Alarm flag: " + (ensemble.al_flag ? "true" : "false");
 	if(ensemble.ecc != FIC_ENSEMBLE::ecc_none)
 		tooltip_text += "\n" "ECC: " + StringTools::IntToHex(ensemble.ecc, 2);
 	if(ensemble.lto != FIC_ENSEMBLE::lto_none)
@@ -1088,7 +1090,16 @@ void DABlinGTK::FICChangeEnsembleEmitted() {
 	if(ensemble.inter_table_id != FIC_ENSEMBLE::inter_table_id_none)
 		tooltip_text += "\n" "International table ID: " + StringTools::IntToHex(ensemble.inter_table_id, 2) + " (" + FICDecoder::ConvertInterTableIDToString(ensemble.inter_table_id) + ")";
 
-	label_ensemble.set_label(label);
+	// indicate active alarm announcement
+	if(ensemble.al_flag) {
+		asw_clusters_t::const_iterator cl_it = ensemble.asw_clusters.find(0xFF);
+		if(cl_it != ensemble.asw_clusters.end() && (cl_it->second.asw_flags & 0x01)) {
+			label += " <span color='white' bgcolor='red'><b> Alarm </b></span>";
+			tooltip_text += "\n" "Alarm SubChId: " + std::to_string(cl_it->second.subchid);
+		}
+	}
+
+	label_ensemble.set_markup(label);
 	frame_label_ensemble.set_tooltip_text(tooltip_text);
 }
 
@@ -1338,6 +1349,12 @@ void DABlinGTK::PADChangeSlideEmitted() {
 		slideshow_window.TryToShow();
 }
 
+void DABlinGTK::PADFileProgressEmitted() {
+//	fprintf(stderr, "### PADFileProgressEmitted\n");
+
+	slideshow_window.UpdateFileProgress(pad_file_progress.Pop());
+}
+
 void DABlinGTK::PADLengthError(size_t /*announced_xpad_len*/, size_t /*xpad_len*/) {
 	fprintf(stderr, "\x1B[31m" "[X-PAD len]" "\x1B[0m" " ");
 }
